@@ -89,59 +89,90 @@ void git2r_tag_init(git_tag *source, SEXP repo, SEXP dest)
  * @param name Name for the tag.
  * @param message The tag message.
  * @param tagger The tagger (author) of the tag
- * @return S3 object of class git_tag
+ * @param force Overwrite existing tag.
+ * @return S3 object of class git_tag or git_commit
  */
-SEXP git2r_tag_create(SEXP repo, SEXP name, SEXP message, SEXP tagger)
+SEXP git2r_tag_create(SEXP repo, SEXP name, SEXP message, SEXP tagger, SEXP force)
 {
     SEXP result = R_NilValue;
-    int error, nprotect = 0;
+    int error, nprotect = 0, overwrite = 0;
     git_oid oid;
     git_repository *repository = NULL;
     git_signature *sig_tagger = NULL;
     git_tag *tag = NULL;
     git_object *target = NULL;
+    git_commit *commit = NULL;
 
     if (git2r_arg_check_string(name))
         git2r_error(__func__, NULL, "'name'", git2r_err_string_arg);
-    if (git2r_arg_check_string(message))
-        git2r_error(__func__, NULL, "'message'", git2r_err_string_arg);
-    if (git2r_arg_check_signature(tagger))
-        git2r_error(__func__, NULL, "'tagger'", git2r_err_signature_arg);
+    if (!Rf_isNull(message)) {
+        if (git2r_arg_check_string(message))
+            git2r_error(__func__, NULL, "'message'", git2r_err_string_arg);
+        if (git2r_arg_check_signature(tagger))
+            git2r_error(__func__, NULL, "'tagger'", git2r_err_signature_arg);
+    }
+    if (git2r_arg_check_logical(force))
+        git2r_error(__func__, NULL, "'force'", git2r_err_logical_arg);
 
     repository = git2r_repository_open(repo);
     if (!repository)
         git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
-    error = git2r_signature_from_arg(&sig_tagger, tagger);
-    if (error)
-        goto cleanup;
-
     error = git_revparse_single(&target, repository, "HEAD^{commit}");
     if (error)
         goto cleanup;
 
-    error = git_tag_create(
-        &oid,
-        repository,
-        CHAR(STRING_ELT(name, 0)),
-        target,
-        sig_tagger,
-        CHAR(STRING_ELT(message, 0)),
-        0);
-    if (error)
-        goto cleanup;
+    if (LOGICAL(force)[0])
+        overwrite = 1;
 
-    error = git_tag_lookup(&tag, repository, &oid);
-    if (error)
-        goto cleanup;
+    if (Rf_isNull(message)) {
+        error = git_tag_create_lightweight(
+            &oid,
+            repository,
+            CHAR(STRING_ELT(name, 0)),
+            target,
+            overwrite);
+        if (error)
+            goto cleanup;
 
-    PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_tag));
-    nprotect++;
-    Rf_setAttrib(result, R_ClassSymbol,
-                 Rf_mkString(git2r_S3_class__git_tag));
-    git2r_tag_init(tag, repo, result);
+        error = git_commit_lookup(&commit, repository, &oid);
+        if (error)
+            goto cleanup;
+
+        PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_commit));
+        nprotect++;
+        Rf_setAttrib(result, R_ClassSymbol,
+                     Rf_mkString(git2r_S3_class__git_commit));
+        git2r_commit_init(commit, repo, result);
+    } else {
+        error = git2r_signature_from_arg(&sig_tagger, tagger);
+        if (error)
+            goto cleanup;
+
+        error = git_tag_create(
+            &oid,
+            repository,
+            CHAR(STRING_ELT(name, 0)),
+            target,
+            sig_tagger,
+            CHAR(STRING_ELT(message, 0)),
+            overwrite);
+        if (error)
+            goto cleanup;
+
+        error = git_tag_lookup(&tag, repository, &oid);
+        if (error)
+            goto cleanup;
+
+        PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_tag));
+        nprotect++;
+        Rf_setAttrib(result, R_ClassSymbol,
+                     Rf_mkString(git2r_S3_class__git_tag));
+        git2r_tag_init(tag, repo, result);
+    }
 
 cleanup:
+    git_commit_free(commit);
     git_tag_free(tag);
     git_signature_free(sig_tagger);
     git_object_free(target);
