@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2015 The git2r contributors
+ *  Copyright (C) 2013-2019 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -22,10 +22,11 @@
 #include <R.h>
 #include <Rinternals.h>
 
-#include "git2.h"
+#include <git2.h>
 
 void git2r_tag_init(git_tag *source, SEXP repo, SEXP dest);
-SEXP git2r_tag_create(SEXP repo, SEXP name, SEXP message, SEXP tagger);
+SEXP git2r_tag_create(SEXP repo, SEXP name, SEXP message, SEXP tagger, SEXP force);
+SEXP git2r_tag_delete(SEXP repo, SEXP name);
 SEXP git2r_tag_list(SEXP repo);
 
 #endif
