@@ -1,5 +1,5 @@
 ## git2r, R bindings to the libgit2 library.
-## Copyright (C) 2013-2015 The git2r contributors
+## Copyright (C) 2013-2018 The git2r contributors
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License, version 2,
@@ -14,7 +14,7 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-library(git2r)
+library("git2r")
 
 ## For debugging
 sessionInfo()
@@ -50,12 +50,12 @@ stopifnot(identical(is_bare(repo), FALSE))
 ## Check result in bare repository
 stopifnot(identical(length(commits(bare_repo)), 1L))
 bare_commit_1 <- commits(bare_repo)[[1]]
-stopifnot(identical(commit_1@sha, bare_commit_1@sha))
-stopifnot(identical(commit_1@author, bare_commit_1@author))
-stopifnot(identical(commit_1@committer, bare_commit_1@committer))
-stopifnot(identical(commit_1@summary, bare_commit_1@summary))
-stopifnot(identical(commit_1@message, bare_commit_1@message))
-stopifnot(!identical(commit_1@repo, bare_commit_1@repo))
+stopifnot(identical(sha(commit_1), sha(bare_commit_1)))
+stopifnot(identical(commit_1$author, bare_commit_1$author))
+stopifnot(identical(commit_1$committer, bare_commit_1$committer))
+stopifnot(identical(commit_1$summary, bare_commit_1$summary))
+stopifnot(identical(commit_1$message, bare_commit_1$message))
+stopifnot(!identical(commit_1$repo, bare_commit_1$repo))
 
 ## Cleanup
 unlink(path_bare, recursive=TRUE)
